/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui.SideNavigationLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.internal.Button");

dojo.require("dojo.i18n"); 
dojo.requireLocalization("net.jazz.ajax.ui.internal", "JAFMessages");

var messages;

dojo.declare("net.jazz.ajax.ui.SideNavigationLayout", [dijit._Widget, dijit._Templated], {

	// summary:
	//		A reusable layout widget which features a collapsible side-
	//		navigation area and a main content area.
	// description:
	//		You can override some of the default styles for this widget; see
	//		resources/ui/SideNavigationLayout.css for details.
	
	addNavigationSection: function(/*Object*/contentNode, /*String?*/title, /* boolean */showSeparator, /* boolean */ startMinimized) {
		// summary:
		//		Adds a new section to the left-hand navigation view containing
		//		the specified contentNode and (optional) title.
		// contentNode:
		//		A Dojo widget or DOM node that contains the content of the
		//		navigation section.
		// title:
		//		The title of the navigation section, if specified, displays at
		//		the top of the section in the ui; otherwise this portion is
		//		hidden.
		// showSeparator:
		//		Indicates if the nav section separator should be shown.
		// since:
		//		0.5
		if (isWidget(contentNode)) {
			var section = new net.jazz.ajax.ui.layout.NavSection(
					{contentWidget: contentNode, heading: title, showSeparator: showSeparator, startMinimized: startMinimized},
					document.createElement("div"));
		} else {
			var section = new net.jazz.ajax.ui.layout.NavSection(
					{contentWidget: {domNode: contentNode}, heading: title, showSeparator: showSeparator, startMinimized: startMinimized},
					document.createElement("div"));
		}
		this._leftNav.appendChild(section.domNode);
		return section;
	},
	/*
	 * addCommand
	 * 
	 * args:
	 * 
	 *  href: string (href will supercede callback if both passed in)
	 * 		OR
	 * 	callback: string OR function
	 * 	
	 * 	imageClass: string
	 * 	buttonText: string
	 * 	boldFirstWord: boolean
	 * 	size: "small" or "large" 
	 *  disabled: boolean
	 */
	
	addCommand: function(args) {
		args["class"] = "net-jazz-ajax-SideNavigationLayout-navCommand";
		var button = new net.jazz.ajax.ui.internal.Button(args);
		this._leftNav.appendChild(button.domNode);
		return button;
	},
	
	setMainContent: function(/*Object*/mainContentDomNode, /*Object*/bannerDomNode) {
		// summary:
		//		Displays the DOM node specified by 'mainContentDomNode' in the
		//		main content area and, if present, the DOM node specified by
		//		'bannerDomNode' above the content area.
		// mainContentDomNode:
		//		The DOM node to display in the main content area.
		// bannerDomNode:
		//		The DOM node to display above the main content area.
		// since:
		//		0.5
		
		// optionally set the meta-content area
		removeChildren(this._metaAreaContainer);
		if (bannerDomNode) {
			this._metaAreaContainer.appendChild(bannerDomNode);
		}
		
		// always set the main content area
		removeChildren(this._contentAreaContainer);
		this._contentAreaContainer.appendChild(mainContentDomNode);
	},

	//
	// Internal methods and properties
	//
	
	templatePath: dojo.moduleUrl("net.jazz.ajax", "ui/internal/templates/SideNavLayout.html"),
	
	postCreate: function() {
		messages = dojo.i18n.getLocalization("net.jazz.ajax.ui.internal", "JAFMessages");
		this._initDimensions();
		this._firstNavSectionLoaded = false;
	},
	
	_initDimensions: function() {
		// summary:
		//		Set the initial state of the elements.
		if (dojo.isIE == 6) {
			this._showHideButton.style.visibility = "hidden";
			this._showHideControl.style.cursor = "normal";
		}
		this._currentLeftNavState = "open";
		this._leftNav.style.width = LEFT_NAV_DEFAULT_WIDTH + "px";
		this._leftNavWidth = LEFT_NAV_DEFAULT_WIDTH;
		switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
	},
	
	_toggleLeftNav: function(e) {
		// disable left nav resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			// description:
			//		If the left-nav is open, close it. If the left-nav is closed,
			//		open it.
			if (dojo.isIE >= 7) {
				this._showHideButton.style.visibility = "hidden";
			}
			dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onStart");
			
			// animate by simultaneously changing margin-right and left
			var args = {
				node: this._leftNav,
				duration: 200,
				properties: {
					left: {
						unit: "px"
					},
					marginRight: {
						unit: "px"
					}
				}
			};
			if (this._currentLeftNavState === "open") {
				// hide
				args.properties.left.end = -this._leftNavWidth;
				args.properties.marginRight.end = -this._leftNavWidth;
				var anim = dojo.animateProperty(args);
				this.connect(anim, "onEnd", this._onHideAnimationEnd);
			}
			else {
				// show
				args.properties.left.end = 0;
				args.properties.marginRight.end = 0;
				var anim = dojo.animateProperty(args);
				this.connect(anim, "onEnd", this._onShowAnimationEnd);
			}
			anim.play();
		}
	},
	
	_onHideAnimationEnd: function() {
		this._currentLeftNavState = "close";
		switchClass(this._showHideButton, "showHideButtonExpanded", "showHideButtonCollapsed");
		if(dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd");
	},
	
	_onShowAnimationEnd: function() {
		this._currentLeftNavState = "open";
		switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
		if(dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd");
	},
	
	//
	// Internal event handlers
	//
	
	_highlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideControl, "net-jazz-ajax-SideNavigationLayout-showHideControl", "net-jazz-ajax-SideNavigationLayout-showHideControl-hover");
		}
	},
	
	_unhighlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideControl, "net-jazz-ajax-SideNavigationLayout-showHideControl-hover", "net-jazz-ajax-SideNavigationLayout-showHideControl");
		}
	},

	_highlightShowHideButton: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideButton, "net-jazz-ajax-SideNavigationLayout-showHideButton", "net-jazz-ajax-SideNavigationLayout-showHideButton-hover");
			switchClass(this._showHideControl, "net-jazz-ajax-SideNavigationLayout-showHideControl-hover", "net-jazz-ajax-SideNavigationLayout-showHideControl");
		}
	},
	
	_unhighlightShowHideButton: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideButton, "net-jazz-ajax-SideNavigationLayout-showHideButton-hover", "net-jazz-ajax-SideNavigationLayout-showHideButton");
			switchClass(this._showHideControl, "net-jazz-ajax-SideNavigationLayout-showHideControl", "net-jazz-ajax-SideNavigationLayout-showHideControl-hover");
		}
	},
	
	_startDragging: function(e) {
		dojo.stopEvent(e);
		dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onStart");
		// disable left nav resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			dojo.addClass(document.body, "net-jazz-ajax-SideNavigationLayout-dragCursor");
			if (this._dragListeners) {
				for (var i = 0; i < this._dragListeners.length; i++) {
					this.disconnect(this._dragListeners[i]);
				}
			}
			this._dragListeners = [
				this.connect(document.body, "onmousemove", "_drag"),
				this.connect(document.body, "onmouseout", "_drag"),
				this.connect(document.body, "onmouseup", "_stopDragging"),
				this.connect(document.body, "ondragstart", "_cancelEvent"),
				this.connect(document.body, "onselectstart", "_cancelEvent")
			];
			this._isDragging = true;
		}
	},

	_stopDragging: function(e) {
		dojo.removeClass(document.body, "net-jazz-ajax-SideNavigationLayout-dragCursor");
		if (this._dragListeners) {
			for (var i = 0; i < this._dragListeners.length; i++) {
				this.disconnect(this._dragListeners[i]);
			}
			delete this._dragListeners;
		}
		var widthpx = this._leftNav.style.width;
		widthpx = widthpx.substring(0, widthpx.length - 2);
		if (widthpx >= LEFT_NAV_MIN_WIDTH) {
			this._leftNavWidth = widthpx;
		}
		this._isDragging = false;
		dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd");
	},
	
	_cancelEvent: function(e) {
		dojo.stopEvent(e);
	},
	
	_drag: function(e) {
		var x = e.pageX - dojo.coords(this._top).x - (SHOW_HIDE_CTRL_WIDTH/2);
		if (x < LEFT_NAV_SNAP_WIDTH) {
			// snap closed
			this._leftNav.style.width = this._leftNavWidth + "px";
			this._leftNav.style.left = -this._leftNavWidth + "px";
			this._leftNav.style.marginRight = -this._leftNavWidth + "px";
			this._currentLeftNavState = "close";
			if (dojo.isIE == 7) {
				// IE7 bug: touch class to trigger re-paint
				this._showHideButton.className = this._showHideButton.className;
			}
			switchClass(this._showHideButton, "showHideButtonExpanded", "showHideButtonCollapsed");
		}
		else if (x < LEFT_NAV_MIN_WIDTH) {
			// prevent further shrinking
			this._leftNav.style.width = LEFT_NAV_MIN_WIDTH + "px";
			this._leftNav.style.left = "0px";
			this._leftNav.style.marginRight = "0px";
			this._currentLeftNavState = "open";
			if (dojo.isIE == 7) {
				// IE7 bug: touch class to trigger re-paint
				this._showHideButton.className = this._showHideButton.className;
			}
			switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
		}
		else if (x > LEFT_NAV_MAX_WIDTH) {
			// prevent further expansion
			this._leftNav.style.width = LEFT_NAV_MAX_WIDTH + "px";
			this._leftNav.style.left = "0px";
			this._leftNav.style.marginRight = "0px";
			this._currentLeftNavState = "open";
			if (dojo.isIE == 7) {
				// IE7 bug: touch class to trigger re-paint
				this._showHideButton.className = this._showHideButton.className;
			}
			switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
		}
		else {
			// slide with mouse
			this._leftNav.style.width = x + "px";
			if (this._leftNav.style.left != "0px") {
				this._leftNav.style.left = "0px";
			}
			if (this._leftNav.style.marginRight != "0px") {
				this._leftNav.style.marginRight = "0px";
			}
			this._currentLeftNavState = "open";
			if (dojo.isIE == 7) {
				// IE7 bug: touch class to trigger re-paint
				this._showHideButton.className = this._showHideButton.className;
			}
			switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
		}
	},
	
	// if a child of the left-nav is clicked, check to see if it was an action
	// to alter the display state of one of the left-nav sections
	// (connecting at the left-nav level to reduce number of event handlers)
	_routeClick: function(e) {
		// walk up the dom tree from the event target
		var node = e.target;
		while (true) {
			// if the event bubbled all the way to the left-nav itself, the
			// event didn't occur on one of the nodes we care about
			if (node === this._leftNav) {
				return;
			}
			// if we hit a node with our special variable, it means that we've
			// hit a hyperlink that wants us to affect the state of the section
			if(node.__jazzNavSection) {
				// stop the event, since the URI is bogus
				dojo.stopEvent(e);
				// extract the action we're going to take from the link's href
				// (something like "minimize", "maximize", etc.)
				var action = node.href.slice(node.href.lastIndexOf("/") + 1);
				// execute the action and return
				node.__jazzNavSection._runAction(action);
				return;
			}
			
			node = node.parentNode;
		}
	}
});

// aliases
var PlatformUI           = net.jazz.ajax.ui.PlatformUI;
var removeChildren       = net.jazz.ajax.core.internal.util.removeChildren;

// constants
var DIGIT_REGEX          = /\d+/;
var CONTENT_AREA_MIN_WIDTH = 200;
var LEFT_NAV_SNAP_WIDTH = 80;
var LEFT_NAV_MIN_WIDTH = 163;
var LEFT_NAV_MAX_WIDTH = 420;
var LEFT_NAV_DEFAULT_WIDTH = 198;
var SHOW_HIDE_CTRL_WIDTH = 8;
var BG_COLOR		     = "#fbfbec";
var TWISTIE_CLOSED   = 0;
var TWISTIE_3QUARTER = 1;
var TWISTIE_OPEN     = 2;

// utility functions

function getElementBottom(domNode) {
	// summary:
	//		returns the y coord for the specified domNode
	var top = dojo.coords(domNode).y;
	var height = dojo.marginBox(domNode).h;
	
	return top + height;
}

//
// Utility functions for NavSection
//

function isWidget(o) {
	return o.buildRendering && o.postCreate;
}

function show() {
	dojo.forEach(arguments, function(x) {
		if (x) {
//			dojo.addClass(x, "net-jazz-ajax-block");
			dojo.removeClass(x, "net-jazz-ajax-hidden");
		}
	});
}

/*
 * Utility function that performs a dojo.addClass and dojo.removeClass in
 * one step, avoiding intermediate flickering and unnecessary painting. Useful
 * for switching back and forth between classes.
 */
function switchClass(/* Node */ node, /* string */ oldClass, /* string */ newClass) {
	// remove old class if present
	var orig = node.className;
	var cls = orig;
	var t = dojo.trim((" " + cls + " ").replace(" " + oldClass + " ", " "));
	if(cls != t){ cls = t; }

	// add new class if present
	if((" "+cls+" ").indexOf(" "+newClass+" ") < 0){
		cls += (cls ? ' ' : '') + newClass;
	}
	
	// apply change if needed
	if (cls != orig) {
		node.className = cls;
	}
}

function hide() {
	dojo.forEach(arguments, function(x) {
		if (x) {
//			dojo.removeClass(x, "net-jazz-ajax-block");
			dojo.addClass(x, "net-jazz-ajax-hidden");
		}
	});
}

//
// Internal widget implementation; do not instantiate directly.
//
dojo.declare("net.jazz.ajax.ui.layout.NavSection", [dijit._Widget, dijit._Templated], {
	
	depth: 0,
	
	addSubSection: function(/*Object*/contentNode, /*String?*/title, onTitleClick, /* string */ iconUrl, /* boolean */ startMinimized, /* domNode */ refNode) {
		// summary:
		//		Adds a new section to the left-hand navigation view containing
		//		the specified contentNode and (optional) title.
		// contentNode:
		//		A Dojo widget or DOM node that contains the content of the
		//		navigation section.
		// title:
		//		The title of the navigation section, if specified, displays at
		//		the top of the section in the ui; otherwise this portion is
		//		hidden.
		// since:
		//		0.5
		if (isWidget(contentNode)) {
			var subSection = new net.jazz.ajax.ui.layout.NavSubSection(
					{contentWidget: contentNode, heading: title, onTitleClick: onTitleClick, depth: this.depth+1, startMinimized: startMinimized},
					document.createElement("div"));
			var navElement = subSection.domNode;
			if (iconUrl) {
				var img = document.createElement("img");
				img.src = iconUrl;
				subSection._icon.appendChild(img);
			}
			else
				dojo.addClass(subSection._icon, "default-container-icon");
			// indend 19px per level
			subSection.navSectionEntry.style.paddingLeft = this.depth * 19 + "px";
		} else {
			var navElement = contentNode;
		}
		if (refNode)
			this._subSections.insertBefore(navElement, refNode);
		else
			this._subSections.appendChild(navElement);
		this._subSections.style.display = "";
		return subSection;
	},
	
	addSectionRow: function(/* string */ title, /* string OR function */ onTitleClick, /* string */ iconUrl, /* domNode */ refNode) {
		var navSectionEntry = document.createElement("a");
		if (dojo.isIE == 6) {
			navSectionEntry.href = "#";
			this.connect(navSectionEntry, "onclick", function(event){
				if (event && event.preventDefault && event.target == navSectionEntry)
					event.preventDefault();
			});
		}
		navSectionEntry.className = "section-entry";
		var link = document.createElement("a");
		if (iconUrl) {
			var icon = document.createElement("div");
			icon.className = "icon";
			link.appendChild(icon);
			var img = document.createElement("img");
			img.src = iconUrl;
			icon.appendChild(img);
		}
		var linkText = document.createElement("div");
		linkText.className = "link-text";
		linkText.innerHTML = title;
		link.appendChild(linkText);
		link.className = "section-container";
		if (dojo.isFunction(onTitleClick)) {
			link.href = "#";
			this.connect(link, "onclick", onTitleClick);
		}
		else 
			link.href = onTitleClick;
		// indend 19px per level
		// plus 15 to account for lack of a twistie image
		navSectionEntry.style.paddingLeft = this.depth * 19 + 15 + "px";
		navSectionEntry.appendChild(link);
		if (refNode)
			this._subSections.insertBefore(navSectionEntry, refNode);
		else
			this._subSections.appendChild(navSectionEntry);
		this._subSections.style.display = "";
		return navSectionEntry;
	},
	
	addArtifactCommand: function(/* domNode */ sectionEntry, /* string */ tooltip, /* string OR function */ onClick, /* string */ iconClass, /* boolean */ hoverCommand) {
		var command = document.createElement("a");
		dojo.addClass(command, "artifactCommand");
		dojo.addClass(command, iconClass);
		if (hoverCommand)
			dojo.addClass(command, "artifactHoverCommand");
		command.title = tooltip;
		if (dojo.isFunction(onClick)) {
			command.href = tooltip ? tooltip : "#";
			this.connect(command, "onclick", onClick);
		} else {
			command.href = onClick;
			this.connect(command, "onclick", this._executeArtifactCommand);
		}
		var sectionContainer = sectionEntry.lastChild;
		while (sectionContainer.nodeType == 3)
			sectionContainer = sectionContainer.previousSibling;
		sectionContainer.insertBefore(command, sectionContainer.lastChild);
		return command;
	},
	
	update: function() {
		if (this._state.hadShowPartial == this._has3Qtr())
			this._runAction(this._state.expansion);
		else if (this._state.expansion == "maximize" && this._has3Qtr())
			this._runAction("showPartial");
		else if (this._state.expansion == "showPartial" && !this._has3Qtr())
			this._runAction("maximize");
	},
	
	//
	// Internal methods and properties
	//
	templatePath: dojo.moduleUrl("net.jazz.ajax", "ui/internal/templates/NavSection.html"),
	
	constructor: function(args) {
		this.showAll = messages.showAll;
		this.showLess = messages.showLess;
	},
	
	postCreate: function() {
		// mixin a reference to this widget into the links that can toggle the
		// display state of the widget (see LeftNavigationLayout#_routeClick,
		// above).
		var self = this;
		dojo.forEach(
			[this._togglerLink, this._showMore, this._showLess],
			function(x) {
				x["__jazzNavSection"] = self;
			});
		this._initSection();
		
		if (this.contentWidget.domNode)
			this._contentContainer.appendChild(this.contentWidget.domNode);
		var initialState = this.startMinimized ? "minimize" : this._has3Qtr() ? "showPartial" : "maximize";
		this._runAction(initialState);
	},
	
	_initSection: function() {
		if(this.showSeparator){
			dojo.addClass(this._togglerLink, "net-jazz-ajax-SideNavigationLayout-navSectionSeparator");
			var leftTrim = document.createElement("div");
			leftTrim.className = "left-trim";
			var rightTrim = document.createElement("div");
			rightTrim.className = "right-trim";
			this._togglerLink.insertBefore(leftTrim, this._togglerLink.firstChild);
			this._togglerLink.insertBefore(rightTrim, this._togglerLink.firstChild);
		}
		if(this.heading) {
			this._togglerLinkText.appendChild(document.createTextNode(this.heading));
		}
		if (!this.showSeparator && !this.heading)
			this._togglerLink.style.display = "none";
	},
	
	_executeArtifactCommand: function (event) {
		dojo.stopEvent(event);
		location.href = event.target.href;
	},
	
	_has3Qtr: function() {
		if (dojo.isFunction(this.contentWidget.hasShowPartial))
			return this.contentWidget.hasShowPartial();
		return false;
	},
	
	_runAction: function(action) {
		
		if (action === "minimize") {
			// in the minimize case, we hide everything
			var nextAction = this._has3Qtr() ? "/showPartial" : "/maximize";
			this._setToggler(TWISTIE_CLOSED, nextAction);
			if(this._has3Qtr() && dojo.isFunction(this.contentWidget.onMinimize))
				this.contentWidget.onMinimize();
			hide(this._contentContainer, this._subSections, this._showMore, this._showLess);
		} else if (action === "maximize") {
			this._setToggler(TWISTIE_OPEN, "/minimize");
			hide(this._showMore);
			show(this._contentContainer, this._subSections);
			if(this._has3Qtr()) {
				show(this._showLess);
				if (dojo.isFunction(this.contentWidget.onMaximize))
					this.contentWidget.onMaximize();
			} else {
				hide(this._showLess);
			}
		} else {
			// if we're in the partial view case, we know we support maximize
			this._setToggler(TWISTIE_3QUARTER, "/maximize");
			hide(this._showLess);
			show(this._contentContainer, this._subSections, this._showMore);
			if (dojo.isFunction(this.contentWidget.onShowPartial))
				this.contentWidget.onShowPartial();
		}
		this._state = {
			expansion: action,
			hadShowPartial: this._has3Qtr()
		};
		
	},
	
	_setToggler: function(currentState, actionUri) {
		this._togglerLink.href = actionUri;
	}
});

dojo.declare("net.jazz.ajax.ui.layout.NavSubSection", net.jazz.ajax.ui.layout.NavSection, {
	
	templatePath: dojo.moduleUrl("net.jazz.ajax", "ui/internal/templates/NavSubSection.html"),
	
	postCreate: function() {
		if (dojo.isIE == 6) {
			/* IE6 anchors must have href specified in order to show hover behaviour.*/
			this.navSectionEntry.href = "#";
			this.connect(this.navSectionEntry, "onclick", function(event){
				if (event && event.preventDefault && event.target == this.navSectionEntry)
					event.preventDefault();
			});
		}
		/* Have to build up the DOM programatically because of bugs
		 * with nested anchors in the template file (strangely enough, 
		 * the child anchors appeared as siblings instead of children 
		 * even though the markup was correct)
		 */
		this._togglerLink = document.createElement("a");
		this._togglerLink.className = "net-jazz-toggle";
		this.navSectionEntry.appendChild(this._togglerLink);
		
		this._titleLink = document.createElement("a");
		this._titleLink.className = "section-container";
		
		this._icon = document.createElement("div");
		this._icon.className = "icon";
		this._titleText = document.createElement("div")
		this._titleText.className = "link-text";
		
		this._titleLink.appendChild(this._icon);
		this._titleLink.appendChild(this._titleText);
		this.navSectionEntry.appendChild(this._titleLink);
		
		this.inherited(arguments);
	},
	_initSection: function() {
		// mixin a reference to this widget into the links that can toggle the
		// display state of the widget (see LeftNavigationLayout#_routeClick,
		// above).
		this._togglerLink["__jazzNavSection"] = this;
		
		if (this.onTitleClick) {
			if (dojo.isFunction(this.onTitleClick))
				this.connect(this._titleLink, "onclick", this.onTitleClick);
			else
				this._titleLink.href = this.onTitleClick;
		} else {
			this._titleLink["__jazzNavSection"] = this;
		}
		if(this.heading) {
			// the span is to avoid a mysterious IE error (unexpected runtime error)
			var span = document.createElement("span");
			span.innerHTML = this.heading;
			this._titleText.appendChild(span);
		}
	},
	
	_setToggler: function(currentState, actionUri) {
		if (currentState == TWISTIE_CLOSED) {
			dojo.addClass(this._togglerLink,"net-jazz-toggle-closed");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-open");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-3quarter");
			hide(this._bottom);
		} else if (currentState == TWISTIE_OPEN) {
			dojo.addClass(this._togglerLink,"net-jazz-toggle-open");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-closed");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-3quarter");
			show(this._bottom);
		} else if (currentState == TWISTIE_3QUARTER) {
			dojo.addClass(this._togglerLink,"net-jazz-toggle-3quarter");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-open");
			dojo.removeClass(this._togglerLink,"net-jazz-toggle-closed");
			show(this._bottom);
		}
		this._togglerLink.href = actionUri;
		if (!this.onTitleClick)
			this._titleLink.href = actionUri;
		// if this isn't done, the chevron will not move on IE
		this._bottom.style.display = "none";
		this._bottom.style.display = "";
	}
});
	
})();