/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.internal.Dialog");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");

(function() {

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var Util = com.ibm.team.repository.web.ui.internal.Util;
	
dojo.declare("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/templates/AboutJazzTeamServer.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.about", "AboutJazzTeamServerMessages");
		this.forUpdatesAndInfoMsg = dojo.string.substitute(this.messages.forUpdatesAndInfo, ["<a href='http://jazz.net' target='_blank'>http://jazz.net</a>"])
	},
	
	postCreate: function() {
		if (!this.dialogTitle) this.dialogTitle = this.messages.aboutJazzTeamServer;
		this.rationalLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/rtl_logo_sm.gif");
		this.rationalLogo.style.marginTop = "32px";
		this.jazzLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/images/jazz-mark.png");
		this.jazzLogo.style.marginTop = "18px";
		this.eclipseLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/BuiltOnEclipse_Logo.png");
		this.eclipseLogo.style.marginTop = "15px";
		this.javaLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/sun-java-comp-logo.png");
		this.licenseKeyManagementLink.href = net.jazz.ajax._contextRoot + "/admin#action=com.ibm.team.repository.admin.manageLicenses";
		if (!Util.userIsAdmin()) {
			this.licenseKeyManagementDiv.style.display = "none";
		}
		var srh = new ServiceResponseHandler(this, "cbGetLicenseInfo", "displayErrorDialog");
		LicenseClient.getLicenseInfo(srh);
		
		var userId = getAuthenticatedContributor().userId;
		var srh2 = new ServiceResponseHandler(this, "cbGetAssignedLicenses", "displayErrorDialog");
		LicenseClient.getAssignedLicenses(srh2, {userId: userId});
	},
	
	cbGetLicenseInfo: function(licenseInfo) {
		this.licenseInfo = licenseInfo;
		this.displayAboutDialog();
	},
	
	cbGetAssignedLicenses: function(calAssignments) {
		this.assignmentIdMap = {};
		var self = this;
		dojo.forEach(calAssignments || [], function(cal) {
			self.assignmentIdMap[cal.id] = true;
		});
		this.displayAboutDialog();
	},
	
	displayAboutDialog: function () {
		// don't execute until we have data from both remote methods
		if(!this.licenseInfo || !this.assignmentIdMap) {
			return;
		}
		var info = this.licenseInfo;
		
		var serverInfo = InitializationData.get("com.ibm.team.repository.service.serverInfoInitializer");
		if (serverInfo.version) {
			this.versionInfo.innerHTML = serverInfo.version;
		} else {
			this.versionInfo.innerHTML = "&nbsp;"
		}
		if (serverInfo.buildId) {
			this.buildIdInfo.innerHTML = serverInfo.buildId;
		} else {
			this.buildIdInfo.innerHTML = "&nbsp;";
		}
		if (info.serverLicense) {
			if (info.serverLicense.productName) {
				this.serverInfo.innerHTML = info.serverLicense.productName;
			} else {
				this.serverInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.editionName) {
				this.editionInfo.innerHTML = info.serverLicense.editionName;
			} else {
				this.editionInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.description) {
				this.descriptionInfo.innerHTML = info.serverLicense.description;
			} else {
				this.descriptionInfo.innerHTML = "&nbsp;";
			}
			if (info.serverLicense.disabledReason) {
				this.licenseKeyInfo.innerHTML = dojo.string.substitute(this.messages.disabled, [info.serverLicense.disabledReason]);
			} else if (info.serverLicense.expirationTime) {
				var expDate = dojo.date.stamp.fromISOString(info.serverLicense.expirationTime);
				var dateStr = dojo.date.locale.format(expDate, {formatLength: "long", selector: "date" });
				this.licenseKeyInfo.innerHTML = dojo.string.substitute(this.messages.trialExpires,  [dateStr]);
			} else {
				this.licenseKeyInfo.innerHTML = this.messages.permanent;
			}
		}
		if (info.types) {
			this.calSection.style.display = "block";
			this.licenseAssignedTo.appendChild(document.createTextNode(dojo.string.substitute(this.messages.licenseAssignedTo, [getAuthenticatedContributor().name])));
			var self = this;
			dojo.forEach(info.types, dojo.hitch(this, function (cal) {
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.appendChild(document.createTextNode(cal.productName || ""));
				tr.appendChild(td);
				td = document.createElement("td");
				if (serverInfo.version) {
					td.appendChild(document.createTextNode(serverInfo.version));
				} else {
					td.appendChild(document.createTextNode(" "));
				}
				tr.appendChild(td);				
				td = document.createElement("td");
				var editionStr = cal.editionName || ""
				if (editionStr && cal.variantName != null) {
					editionStr += " (" + cal.variantName + ")";
				}
				td.appendChild(document.createTextNode(editionStr));
				tr.appendChild(td);
				td = document.createElement("td");
				td.appendChild(document.createTextNode(cal.name || ""));
				tr.appendChild(td);
				td = document.createElement("td");
				td.className = "centered";
				var isAssignedToUser = self.assignmentIdMap[cal.id];
				td.appendChild(document.createTextNode(isAssignedToUser ? this.messages.yes : this.messages.no));
				tr.appendChild(td);
				this.installedCALBody.appendChild(tr);
			}));
		}
		this.showDialog();
	},
	
	displayErrorDialog: function(err) {
		if(this.errorIsDiplayed) {
			return;
		}
		this.errorIsDiplayed = true;
		var container = document.createElement("div");
		container.style.padding = "1em";
		container.className = "com-ibm-team-repository-web-ui-internal-about";
		var div = document.createElement("div");
		div.appendChild(document.createTextNode(this.messages.errorFetchingInformation));
		div.className = "errorMsg";
		container.appendChild(div);
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: container,
			primaryTitle: this.messages.aboutJazzTeamServer,
			width: "600px"
		});
	},
	
	showDialog: function () {
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: "800px"
		});
	}
});

})();