/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");


(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;

var DB_SERVICE_NAME = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
var DB_VENDOR_NAME = "db.vendor";
var DB_TYPE_NAME = "db.type";
var JDBC_TYPE_VALUE = "JDBC";
var VENDOR_LICENSE_CHECK_PREFIX = "allows." + DB_VENDOR_NAME + ".";
	
var DISPLAY_PROPS = [];
DISPLAY_PROPS.J2EE = [
	"db.j2ee.dataSource"
];
DISPLAY_PROPS.JDBC = [
	"db.jdbc.password",
	"db.jdbc.location"
];

var messages;

/*          	TEST/REVERT/NEXT/EDIT
LOADING        		N   N   N   N
TESTING        		N   N   N   N
PENDING_OK 	      	Y   ?   N   Y 
PENDING_ERR 	    N   Y   N   Y
PENDING_WARN  	    Y   Y   N   Y
CONNECTED_OK        N   Y   Y   Y
CONNECTED_ERR       N   Y   N   Y
CONNECTED_WARN      N   Y   Y   Y  
*/
var STATUS = {
	LOADING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS",
		message: 'loading'},  
	TESTING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS", 
		message: 'testing'},  
	PENDING_OK: {test: true, revert: true, next: false, edit: true, code: "INFO", 
		message: 'pendingOk'},  
	PENDING_ERR: {test: false, revert: true, next: false, edit: true, code: "ERROR",
		message: 'pendingErr'}, 
	PENDING_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
		message: 'pendingWarn'}, 
	CONNECTED_OK: {test: true, revert: true, next: true, edit: true, code: "OK",  
		message: 'connectedOk'},
	CONNECTED_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: 'connectedErr'}, 
	CONNECTED_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
		message: 'connectedWarn'}, 
	SAVE_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: 'saveErr'}
};

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase",  [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigureDatabase.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");
		var JAZZ_ROOT = net.jazz.ajax._contextRoot;
		var adminLink = "<a href='" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.manageLicenses'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ adminLink, "</a>" ]);
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase.superclass.postCreate.apply(this);
		this._navigation = this.wizardNav;
		this._navigation.enablePrevious(true);		
		dojo.connect(this._navigation, "requestNextApproval", this, function() {
			this._navigation.enableNext(false);	
			var invokedSave = this._saveHandler();
			if (!invokedSave) {
				this._navigation.processNext();
				this._navigation.enableNext(true);	
			}
		});
		// NOTE: no changes require server restart 
		dojo.connect(this, "_submitConfigCallback", this, function() {
			this._initialLoadedState = STATUS.CONNECTED_OK;
			this._setState(STATUS.CONNECTED_OK);
			this._navigation.processNext();
			this._navigation.enableNext(true);	
		}); 
		dojo.connect(this, "_submitConfigError", this, function(){
			this._setState(STATUS.SAVE_ERR);
		}); 
		this._setState(STATUS.LOADING);
		this._displayVendorsAllowedByLicense();
		this._initializeContent();
	},
		
	// INITAL LOAD METHODS
	_initializeContent: function () {
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: DB_SERVICE_NAME
			},
			enableEdits: true,
			displayIn: this._hiddenTableBody,
			descriptionMode: "table", 
			initialMode: "edit"
		}
		this._dbConfig = new ServerPropertyService(serviceConfigProps);
		dojo.connect(this._dbConfig, "_markChanged", this, function() {
			this._setState(STATUS.PENDING_OK);
		});
		dojo.connect(this._dbConfig, "_checkUnchanged", this, function () {
				if(this._dbConfig._changed) {
					this._setState(STATUS.PENDING_OK);
				} else {
					this._setState(this._initialLoadedState);
				}
			});
		dojo.connect(this._dbConfig, "_markInvalid", this, "_handleInvalidInput");
		dojo.connect(this._dbConfig, "_show", this, "_initializePropertiesAfterLoaded");
	},
	
	_displayVendorsAllowedByLicense: function () {
		if (licenseAllowsVendor("DB2")) {
			var newOption = new Option("DB2", "DB2");
			this._selectVendor.options[this._selectVendor.options.length] = newOption;
		} else {
			this._displayLicenseLimitedMessage();
		}
		if (licenseAllowsVendor("DERBY")) {
			var newOption = new Option("Derby", "DERBY");
			this._selectVendor.options[this._selectVendor.options.length] = newOption;
		} 
		if (licenseAllowsVendor("ORACLE")) {
			var newOption = new Option("Oracle", "ORACLE");
			this._selectVendor.options[this._selectVendor.options.length] = newOption;
		} else {
			this._displayLicenseLimitedMessage();
		}
		if (licenseAllowsVendor("SQLSERVER")) {
			var newOption = new Option("SQL Server", "SQLSERVER");
			this._selectVendor.options[this._selectVendor.options.length] = newOption;
		} else {
			this._displayLicenseLimitedMessage();
		}
	},

	_displayLicenseLimitedMessage: function() {
		dojo.toggleClass(this._licenseLimitedMessage, "noDisplay", false);
	}, 

	_initializePropertiesAfterLoaded: function () {
		this._initializeHiddenProperty(DB_VENDOR_NAME);
		this._initializeHiddenProperty(DB_TYPE_NAME);
		
		this._initializePropertyDisplay(this._jdbcEditPropertiesBody, DISPLAY_PROPS.JDBC);
		this._initializePropertyDisplay(this._j2eeEditPropertiesBody, DISPLAY_PROPS.J2EE);
		
		// create sections for advanced vendor props 
		//this._initializePropertyDisplay(this._db2EditPropertiesBody, DISPLAY_PROPS.DB2);
		//this._initializePropertyDisplay(this._db2EditPropertiesBody, DISPLAY_PROPS.ORACLE);
		//this._initializePropertyDisplay(this._db2EditPropertiesBody, DISPLAY_PROPS.SQLSERVER);
		
		this._synchronizeSelectValue();
		this._displayRelevantSections();
		this._testConnection();
	},	
	
	_initializeHiddenProperty: function (propertyName) {
		var targetProperty = this._dbConfig.getServerPropertyElement(propertyName);
		dojo.connect(targetProperty, "_markInvalid", this, function (msg) {
			this._handleInvalidInput(msg);
			this._showConnectionErrors([msg]);
		});
	},
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {
		
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this._dbConfig.getServerPropertyElement(displayProps[i]);
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
	
	// HANDLE USER INTERACTION METHODS
	 onInnerButtonAreaClick: function(e) {
		var button = e.target;
		if (button === this._revertButton) {
			this._revert();
			this._displayRelevantSections();
		} else if (button === this._testConnectionButton){
			this._testConnection();
		} 
	}, 
	
	_handleVendorSelectChange: function (event) {
		this._handleSelectChange(event, DB_VENDOR_NAME);
	},
	
	_handleTypeSelectChange: function(event){
		this._handleSelectChange(event, DB_TYPE_NAME);
	}, 
	
	_handleSelectChange: function (event, propertyName) {
		var newSelectValue = event.target.value;
		this._handleChange(newSelectValue, propertyName);
	},
	
	_handleChange: function (newValue, propertyName) {
		var targetProperty = this._dbConfig.getServerPropertyElement(propertyName);
		
		targetProperty._editValueNode.value = newValue;
		
		if (targetProperty._savedValue != newValue) {
			targetProperty._markChanged();
		} else {
			targetProperty._markUnchanged();
		}
		targetProperty._checkValidity();
		this._displayRelevantSections();
	},
	
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this._setState(STATUS.PENDING_ERR);
		} else {
			this._setState(STATUS.PENDING_WARN);
		}
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		this._setState(STATUS.TESTING);
		var srh = new ServiceResponseHandler(this, "_testConnectionCallback", "_testConnectionError");
		this._dbConfig.testRobustValidity(srh);
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
	
		if (msg != null) {
			connected = STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = STATUS.CONNECTED_ERR;
				}
			}
		} else {
			connected = STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected);
		if (msg != null) {
			this._showConnectionErrors(msg);
		}
	},
	
	_testConnectionError: function(errorObj) {
		this._setState(STATUS.CONNECTED_ERR);
		this._showConnectionErrors(messages.connectionError + errorObj.message);
	},
	
	_revert: function() {
		this._dbConfig.restoreSaved();
		this._synchronizeSelectValue();
		this._displayRelevantSections();
		this._setState(this._initialLoadedState);
	},

	// HANDLE STATE METHODS
	_setState: function(newState) {
		var state = newState;
		// check required for initial load before service properties are loaded
		if (this._dbConfig != null) {
			this._allowPropertyEdit(state.edit); 
		}
		var enableRevert = state.revert;
		if (enableRevert && !this._dbConfig._changed) {
			enableRevert = false;
		}
		this._revertButton.disabled = (!enableRevert);
		this._testConnectionButton.disabled = (!state.test);
		this._navigation.enableNext(state.next);
		this._showConnectionStatus(state.code, messages[state.message]);
	},
			
	// UPDATE VIEW METHODS	
	_displayRelevantSections: function() {
		// KMB TODO need to clear out values on hide
		if (this._dbConfig.getEditedValueForProperty(DB_TYPE_NAME) === JDBC_TYPE_VALUE){
			this._j2eePropertiesSection.style.display = "none";
			this._jdbcPropertiesSection.style.display = "block";
			//KMB TODO show vendor props if jdbc
		} else {
			this._j2eePropertiesSection.style.display = "block";
			this._jdbcPropertiesSection.style.display = "none";
		}
	}, 
	
	_allowPropertyEdit: function (allowEdits){
		var props = this._dbConfig._properties;
		for (var i=0; i < props.length; i++) {
			props[i]._editValueNode.disabled = !allowEdits;
		};	 
		this._selectVendor.disabled = !allowEdits;
		this._selectType.disabled = !allowEdits;
	},
	
	_synchronizeSelectValue: function() {
		var vendorProp = this._dbConfig.getServerPropertyElement(DB_VENDOR_NAME);
		if (vendorProp._editValueNode.value === "DERBY" && !licenseAllowsVendor("DERBY")) {
			this._handleChange("DB2", DB_VENDOR_NAME);
		}
		this._selectVendor.value = vendorProp._editValueNode.value;
		var typeProp = this._dbConfig.getServerPropertyElement(DB_TYPE_NAME);
		this._selectType.value = typeProp._editValueNode.value;
	},
	
	_showConnectionStatus: function(code, statusMsg) {
		dojox.data.dom.removeChildren(this._stepStatus);
		
		var status = document.createElement("div");
		var indication = document.createElement("div");
		status.className = "stepStatus " + code;
		indication.className = "stepStatus statusMsg " + code;
		
		var statusImage = document.createElement("div");
		statusImage.className = "statusImage " + code+ "img";
		status.appendChild(statusImage);
		
		var message = document.createElement("div");
		dojo.addClass(message,"statusMsg");
		message.innerHTML = statusMsg;
		indication.innerHTML = statusMsg;
		status.appendChild(message);
		
		this._stepStatus.appendChild(status);
		this._setStatusIndication(code, indication);
	}, 
	
	_setStatusIndication: function (code, node) {
		if (code != "INFO") {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(node);
			if (code != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}
		}
	}, 
	
	_showConnectionErrors: function(msg) {
		for (var i = 0; i < msg.length; i++) {
			var message = document.createElement("div");
			message.innerHTML = msg[i].message;
			message.className = "stepStatus validationDetails " + msg[i].code;
			this._stepStatus.appendChild(message);
		}
	}, 
		
	// SUPER ConfigUtility METHODS	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		if (this._dbConfig._changed) {
			var changeObj = this._dbConfig._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		return {
			"changes": changes,
			"properties": properties
		};
	}
	
});

function licenseAllowsVendor(vendorName) {
	return (dojo.getObject(VENDOR_LICENSE_CHECK_PREFIX + vendorName) != null && 
			dojo.getObject(VENDOR_LICENSE_CHECK_PREFIX + vendorName));
}

})();